'From Squeak3.2gamma of 15 January 2002 [latest update: #4743] on 14 July 2002 at 2:21:23 pm'!
Object subclass: #ConnectionHandler
	instanceVariableNames: 'portNumber socketClass process priority handlerBlock lastError socketsToDestroy '
	classVariableNames: 'Registry '
	poolDictionaries: ''
	category: 'Kom-kernel'!

!ConnectionHandler commentStamp: '<historical>' prior: 0!
Kom48 (Comanche beta 4.8)
bolot 4/2/2001 18:44

ConnectionHandler

Comment from kom46:
- also see ConnectionQueue
- does not keep the queue of open sockets
- launches the applications/modules immediately
  upon accepting a connection!

ConnectionHandler class
	instanceVariableNames: ''!

!ConnectionHandler methodsFor: 'accessing' stamp: 'bolot 1/11/2001 15:01'!
portNumber
	^portNumber! !

!ConnectionHandler methodsFor: 'accessing' stamp: 'bolot 1/11/2001 15:01'!
priority
	^priority! !

!ConnectionHandler methodsFor: 'accessing' stamp: 'bolot 1/11/2001 15:01'!
process
	process == nil ifTrue: [
		self initializeProcessAtPriority: self priority.
	].
	^process! !

!ConnectionHandler methodsFor: 'accessing' stamp: 'svp 12/1/1999 17:58'!
socketsToDestroy

	socketsToDestroy == nil ifTrue: [
		socketsToDestroy _ OrderedCollection new.
	].
	^socketsToDestroy! !

!ConnectionHandler methodsFor: 'initialize-release' stamp: 'bolot 12/16/2000 18:27'!
destroySockets
	socketsToDestroy ~~ nil ifTrue: [ 
		socketsToDestroy do: [ :ea | ea destroy ].
	].
	socketsToDestroy _ OrderedCollection new! !

!ConnectionHandler methodsFor: 'initialize-release' stamp: 'svp 12/1/1999 18:52'!
finalize

	"Like #stop, but without the #unregister"

	process ~~ nil ifTrue: [
		process terminate.
	].
	process _ nil.
	self destroySockets.


! !

!ConnectionHandler methodsFor: 'initialize-release' stamp: 'bolot 12/16/2000 18:38'!
initializeOnPort: aNumber handler: aBlock priority: pNumber
	portNumber _ aNumber.
	handlerBlock _ aBlock.
	priority _ pNumber.
	socketsToDestroy _ OrderedCollection new! !

!ConnectionHandler methodsFor: 'initialize-release' stamp: 'bolot 12/16/2000 18:45'!
initializeProcessAtPriority: pNumber
	process _ [ self pvtListenLoop ] newProcess.
	process priority: pNumber! !

!ConnectionHandler methodsFor: 'printing' stamp: 'bolot 1/11/2001 15:02'!
printOn: aStream
	aStream nextPutAll: (self class name asString, ' port: ', self portNumber asString).
	self isRunning ifTrue: [
		aStream nextPutAll: ' [running]'
	] ifFalse:[
		aStream nextPutAll: ' [stopped]'
	]! !

!ConnectionHandler methodsFor: 'queries' stamp: 'svp 12/1/1999 17:51'!
isRunning

	^process ~~ nil! !

!ConnectionHandler methodsFor: 'registry' stamp: 'svp 12/1/1999 18:46'!
register

	^self class register: self! !

!ConnectionHandler methodsFor: 'registry' stamp: 'svp 12/1/1999 18:46'!
unregister

	^self class unregister: self! !

!ConnectionHandler methodsFor: 'starting and stopping' stamp: 'bolot 12/16/2000 18:42'!
start
	self stop.
	self process resume.
	self register! !

!ConnectionHandler methodsFor: 'starting and stopping' stamp: 'bolot 12/18/2000 23:57'!
stop
	self unregister.
	process ~~ nil ifTrue: [
		process terminate.
	].
	process _ nil.

	"We do the following in a timely manner instead of relying on
	finalization because we may want to turn right around and start
	listening again.  If an old socket is still around an listening, it may
	interfere with a new one's ability to hear."

	self destroySockets! !

!ConnectionHandler methodsFor: 'private' stamp: 'bolot 2/22/2001 16:58'!
newListener
	| listener |

	self flag: #todo. "listener _ self socketClass newTCP."
	listener _ Socket newTCP.

	"backlog is how many simultaneous connections to accept at the same time"
	listener listenOn: portNumber backlogSize: 10.
	^listener! !

!ConnectionHandler methodsFor: 'private' stamp: 'bolot 12/19/2000 00:24'!
pvtListenLoop
	[
		self pvtNewListenLoop
	] on: Error do: [ :ex |
		process _ nil.
		self destroySockets.
	]! !

!ConnectionHandler methodsFor: 'private' stamp: 'bolot 12/19/2000 00:24'!
pvtNewListenLoop
	| socket listener |

	listener _ self newListener.
	self socketsToDestroy add: listener.	

	"If the listener is not valid then the we cannot use the
	BSD style accept() mechanism."
	listener isValid ifFalse: [^self pvtOldListenLoop].
	[ true ] whileTrue: [
		socket _ listener waitForAcceptUntil: (Socket deadlineSecs: 10).
		(socket notNil and: [ socket isConnected ])
			ifTrue: [
				handlerBlock value: socket.
				socket _ nil]
			ifFalse: [
				socket notNil ifTrue: [socket destroy].
				listener isValid
					ifFalse: [(Delay forMilliseconds: 100) wait.
						listener destroy.
						listener _ self newListener].
			].
	]! !

!ConnectionHandler methodsFor: 'private' stamp: 'bolot 12/19/2000 00:24'!
pvtOldListenLoop
	"see comment on ConnectionQueue>>oldStyleListenLoop"
	| socket |
	socket _ nil.

	[ true ] whileTrue: [
		(socket == nil) ifTrue: [
			"try to create a new socket for listening"
			socket _ Socket createIfFail: [ nil ]].

		(socket == nil)
		ifTrue: [(Delay forMilliseconds: 100) wait]
		ifFalse: [
			socket isUnconnected ifTrue: [ socket listenOn: portNumber ].
			socket waitForConnectionUntil: (Socket deadlineSecs: 10).
			socket isConnected
				ifTrue: ["connection established"
					"mutex critical: [" handlerBlock value: socket "]".
					socket _ nil]
				ifFalse: [
					socket isWaitingForConnection
						ifFalse: [socket destroy. socket _ nil]].
		].
	]! !


!ConnectionHandler class methodsFor: 'instance creation' stamp: 'bolot 12/16/2000 18:43'!
onPort: aNumber handler: aBlock priority: pNum
	^self new
		initializeOnPort: aNumber
		handler: aBlock
		priority: pNum! !

!ConnectionHandler class methodsFor: 'registry' stamp: 'svp 12/1/1999 18:49'!
register: anObject

	self registry add: anObject! !

!ConnectionHandler class methodsFor: 'registry' stamp: 'svp 12/1/1999 18:49'!
registry

	Registry == nil ifTrue: [
		Registry := WeakRegistry new
	].
	^Registry! !

!ConnectionHandler class methodsFor: 'registry' stamp: 'svp 12/1/1999 18:50'!
unregister: anObject

	self registry remove: anObject ifAbsent: [ ]! !

