'From Squeak3.2gamma of 15 January 2002 [latest update: #4743] on 14 July 2002 at 2:18:47 pm'!
"Change Set:		WeakMessageSend-nk
Date:			25 April 2002
Author:			Ned Konz

This is a simple MessageSend where the receiver and arguments are held onto weakly."!

Object weakSubclass: #WeakMessageSend
	instanceVariableNames: 'selector shouldBeNil arguments '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Objects'!
Error subclass: #WeakMessageSendError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'System-Exceptions Kernel'!
TestCase subclass: #WeakMessageSendTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Kernel-Tests-EventManager'!

!Object methodsFor: 'testing' stamp: 'nk 4/25/2002 08:04'!
isMessageSend
	^false
! !


!MessageSend methodsFor: 'testing' stamp: 'nk 4/25/2002 08:04'!
isMessageSend
	^true
! !


!WeakMessageSend methodsFor: 'accessing' stamp: 'nk 4/25/2002 09:54'!
arguments
	^arguments ifNil: [ Array new ]
! !

!WeakMessageSend methodsFor: 'accessing' stamp: 'nk 4/25/2002 09:38'!
arguments: anArray
	arguments _ WeakArray withAll: anArray.
	"no reason this should be a WeakArray"
	shouldBeNil _ Array withAll: (anArray collect: [ :ea | ea isNil ]).
! !

!WeakMessageSend methodsFor: 'accessing' stamp: 'nk 4/25/2002 07:54'!
receiver
	^self at: 1
! !

!WeakMessageSend methodsFor: 'accessing' stamp: 'nk 4/25/2002 07:54'!
receiver: anObject
	self at: 1 put: anObject
! !

!WeakMessageSend methodsFor: 'accessing' stamp: 'nk 4/25/2002 07:54'!
selector
	^selector
! !

!WeakMessageSend methodsFor: 'accessing' stamp: 'nk 4/25/2002 07:55'!
selector: aSymbol
	selector _ aSymbol
! !

!WeakMessageSend methodsFor: 'comparing' stamp: 'nk 4/25/2002 08:05'!
= anObject
	"Compare equal to equivalent MessageSend"
	^ anObject isMessageSend
		and: [self receiver == anObject receiver
		and: [selector == anObject selector
		and: [(Array withAll: arguments) = (Array withAll: anObject arguments)]]]
! !

!WeakMessageSend methodsFor: 'comparing' stamp: 'nk 4/25/2002 09:31'!
hash
	"work like MessageSend>>hash"
	^self receiver hash bitXor: selector hash
! !

!WeakMessageSend methodsFor: 'converting' stamp: 'nk 4/25/2002 09:33'!
asMessageSend
	^MessageSend receiver: self receiver selector: selector arguments: (Array withAll: self arguments) 
! !

!WeakMessageSend methodsFor: 'evaluating' stamp: 'nk 4/25/2002 10:03'!
value
	^arguments isNil
		ifTrue: [ self ensureReceiver.
			self receiver perform: selector ]
		ifFalse: [ self ensureReceiverAndArguments.
			self receiver perform: selector withArguments: (Array withAll:  arguments) ]
! !

!WeakMessageSend methodsFor: 'evaluating' stamp: 'rw 4/27/2002 07:41'!
valueWithArguments: anArray
	self ensureReceiverAndArguments.
	^ self receiver 
		perform: selector 
		withArguments: (self collectArguments: anArray)! !

!WeakMessageSend methodsFor: 'evaluating' stamp: 'nk 4/25/2002 09:50'!
valueWithEnoughArguments: anArray
	"call the selector with enough arguments from arguments and anArray"
	| args |
	self ensureReceiverAndArguments.
	args _ Array new: selector numArgs.
	args replaceFrom: 1
		to: ( arguments size min: args size)
		with: arguments
		startingAt: 1.
	args size > arguments size ifTrue: [
		args replaceFrom: arguments size + 1
			to: (arguments size + anArray size min: args size)
			with: anArray
			startingAt: 1.
	].
	^ self receiver perform: selector withArguments: args
! !

!WeakMessageSend methodsFor: 'printing' stamp: 'nk 4/25/2002 09:36'!
printOn: aStream

        aStream
                nextPutAll: self class name;
                nextPut: $(.
        selector printOn: aStream.
        aStream nextPutAll: ' -> '.
        self receiver printOn: aStream.
        aStream nextPut: $)
! !

!WeakMessageSend methodsFor: 'testing' stamp: 'nk 4/25/2002 08:04'!
isMessageSend
	^true
! !

!WeakMessageSend methodsFor: 'tiles' stamp: 'nk 4/25/2002 09:34'!
asTilesIn: playerClass
	^self asMessageSend asTilesIn: playerClass
! !

!WeakMessageSend methodsFor: 'tiles' stamp: 'nk 4/25/2002 09:34'!
asTilesIn: playerClass globalNames: makeSelfGlobal
	^self asMessageSend asTilesIn: playerClass globalNames: makeSelfGlobal
! !

!WeakMessageSend methodsFor: 'tiles' stamp: 'nk 4/25/2002 09:35'!
stringFor: anObject
	^self asMessageSend stringFor: anObject
! !

!WeakMessageSend methodsFor: 'private' stamp: 'nk 4/25/2002 09:49'!
collectArguments: anArgArray
	"Private"
    | staticArgs |
    staticArgs := self arguments.
    ^(anArgArray size = staticArgs size)
        ifTrue: [anArgArray]
        ifFalse:
            [(staticArgs isEmpty
                ifTrue: [ staticArgs := Array new: selector argumentCount]
                ifFalse: [staticArgs copy] )
                    replaceFrom: 1
                    to: (anArgArray size min: staticArgs size)
                    with: anArgArray
                    startingAt: 1]
! !

!WeakMessageSend methodsFor: 'private' stamp: 'nk 4/25/2002 09:51'!
ensureArguments
	"Make sure that my arguments haven't gone away"
	arguments ifNotNil: [
		arguments with: shouldBeNil do: [ :arg :flag |
			arg ifNil: [ flag ifFalse: [ WeakMessageSendError signal: 'argument has been collected' ]]
		]
	]
! !

!WeakMessageSend methodsFor: 'private' stamp: 'nk 4/25/2002 09:48'!
ensureReceiver
	"Make sure that my receiver hasn't gone away"
	self receiver ifNil: [ WeakMessageSendError signal: 'receiver has been collected' ]
! !

!WeakMessageSend methodsFor: 'private' stamp: 'nk 4/25/2002 09:51'!
ensureReceiverAndArguments

	"Make sure that my receiver hasn't gone away"
	self receiver ifNil: [ WeakMessageSendError signal: 'receiver has been collected' ].

	"Make sure that my arguments haven't gone away"
	arguments ifNotNil: [
		arguments with: shouldBeNil do: [ :arg :flag |
			arg ifNil: [ flag ifFalse: [ WeakMessageSendError signal: 'argument has been collected' ]]
		]
	]
! !


!WeakMessageSend class methodsFor: 'instance creation' stamp: 'nk 4/25/2002 10:00'!
new
	^self new: 1
! !

!WeakMessageSend class methodsFor: 'instance creation' stamp: 'nk 4/25/2002 09:37'!
receiver: anObject selector: aSymbol
	^ self receiver: anObject selector: aSymbol arguments: #()
! !

!WeakMessageSend class methodsFor: 'instance creation' stamp: 'nk 4/25/2002 09:37'!
receiver: anObject selector: aSymbol argument: aParameter
	^ self receiver: anObject selector: aSymbol arguments: (Array with: aParameter)
! !

!WeakMessageSend class methodsFor: 'instance creation' stamp: 'nk 4/25/2002 09:37'!
receiver: anObject selector: aSymbol arguments: anArray
	^ self new
		receiver: anObject;
		selector: aSymbol;
		arguments: anArray
! !


!WeakMessageSendTestCase methodsFor: 'as yet unclassified' stamp: 'nk 4/25/2002 10:12'!
testNoArguments
	| m |
	m _ WeakMessageSend
		receiver: true
		selector: #yourself.
	self should: [ m value  ].
! !

!WeakMessageSendTestCase methodsFor: 'as yet unclassified' stamp: 'nk 4/25/2002 10:10'!
testOneArgument
	| m |
	m _ WeakMessageSend
		receiver: Array
		selector: #with:
		argument: 1.
	Smalltalk garbageCollectMost.
	self should: [ m value  = { 1 } ].
! !

!WeakMessageSendTestCase methodsFor: 'as yet unclassified' stamp: 'nk 4/25/2002 10:13'!
testOneArgumentWithGC
	| m |
	m _ WeakMessageSend
		receiver: Array
		selector: #with:
		arguments: { Object new }.
	Smalltalk garbageCollectMost.
	self should: [ m value ]
		raise: WeakMessageSendError.
! !

!WeakMessageSendTestCase methodsFor: 'as yet unclassified' stamp: 'nk 4/25/2002 10:14'!
testReceiverWithGC
	| m |
	m _ WeakMessageSend
		receiver: Object new
		selector: #isNil.
	Smalltalk garbageCollectMost.
	self should: [ m value ]
		raise: WeakMessageSendError.
! !

!WeakMessageSendTestCase methodsFor: 'as yet unclassified' stamp: 'nk 4/25/2002 10:11'!
testTwoArguments
	| m |
	m _ WeakMessageSend
		receiver: Array
		selector: #with:with:
		arguments: { 1 . 2 }.
	Smalltalk garbageCollectMost.
	self should: [ m value = { 1 . 2 } ].
! !


!WeakMessageSend reorganize!
('accessing' arguments arguments: receiver receiver: selector selector:)
('comparing' = hash)
('converting' asMessageSend)
('evaluating' value valueWithArguments: valueWithEnoughArguments:)
('printing' printOn:)
('testing' isMessageSend)
('tiles' asTilesIn: asTilesIn:globalNames: stringFor:)
('private' collectArguments: ensureArguments ensureReceiver ensureReceiverAndArguments isAnyArgumentGarbage isReceiverGarbage isReceiverOrAnyArgumentGarbage)
!

